#!/bin/bash

# Script name: dropshell.sh
# Description: A basic shell script for common operations

# Colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
NC='\033[0m' # No Color

# Source version information
SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"
source "$SCRIPT_DIR/version.sh"

# Source configuration
if [ -f "/etc/dropshell.conf" ]; then
    source "/etc/dropshell.conf"
else
    echo -e "${RED}Error: Configuration file not found${NC}"
    echo "Please run the installation script first"
    exit 1
fi

# Check if USER_DEFINITIONS is set
if [ -z "$USER_DEFINITIONS" ]; then
    echo -e "${RED}Error: USER_DEFINITIONS not set${NC}"
    echo "Please run the installation script first"
    exit 1
fi

# Function to print usage
print_usage() {
    echo "Usage: $0 <command> [options]"
    echo
    echo "Commands:"
    echo "  help      - Show this help message"
    echo "  version   - Show version information"
    echo "  status    - Check system status"
    echo "  servers   - List configured servers"
    echo
    echo "Options:"
    echo "  -v, --verbose    Enable verbose output"
}

# Function to print version
print_version() {
    get_version_info
}

# Function to check status
check_status() {
    echo -e "${GREEN}System Status:${NC}"
    echo "Date: $(date)"
    echo "Uptime: $(uptime)"
    echo "Memory Usage:"
    free -h
    echo
    echo "Disk Usage:"
    df -h /
}

# Function to list servers from _server.env files
list_servers() {
    local servers_dir="/opt/dropshell/user/servers"
    local max_name_len=0
    local max_addr_len=0
    local servers=()

    # First pass: collect data and find max lengths
    for server_dir in "$servers_dir"/*/; do
        if [ -f "${server_dir}_server.env" ]; then
            local server_name=$(basename "$server_dir")
            local ssh_address=$(grep '^SSH_ADDRESS=' "${server_dir}_server.env" | cut -d'=' -f2-)
            
            # Update max lengths
            [ ${#server_name} -gt $max_name_len ] && max_name_len=${#server_name}
            [ ${#ssh_address} -gt $max_addr_len ] && max_addr_len=${#ssh_address}
            
            servers+=("$server_name|$ssh_address")
        fi
    done

    # Add padding for headers
    max_name_len=$((max_name_len > 4 ? max_name_len : 4))
    max_addr_len=$((max_addr_len > 7 ? max_addr_len : 7))

    # Print header
    printf "%-${max_name_len}s | %-${max_addr_len}s\n" "Name" "Address"
    printf "%-${max_name_len}s-+-%-${max_addr_len}s\n" "$(printf '%*s' $max_name_len '' | tr ' ' '-')" "$(printf '%*s' $max_addr_len '' | tr ' ' '-')"

    # Print server rows
    for server in "${servers[@]}"; do
        IFS='|' read -r name addr <<< "$server"
        printf "%-${max_name_len}s | %-${max_addr_len}s\n" "$name" "$addr"
    done
}

# Main script logic
case "$1" in
    "help"|"-h"|"--help")
        print_usage
        ;;
    "version"|"-V"|"--version")
        print_version
        ;;
    "status")
        check_status
        ;;
    "servers")
        list_servers
        ;;
    "")
        echo -e "${RED}Error: No command provided${NC}"
        print_usage
        exit 1
        ;;
    *)
        echo -e "${RED}Error: Unknown command '$1'${NC}"
        print_usage
        exit 1
        ;;
esac

exit 0